import queryString from "query-string";
import React, { useEffect } from "react";
import { useLocation, useRoutes } from "react-router-dom";
import { loadingId, pageInit } from "../../../config/index.js";
import { reduxConnect } from "../../connect.js";
import { getInitialData, getMatchedComponent } from "../../libs/metaCollector.js";
import getMatchedBranch from "../../libs/getMatchedBranch.js";
import getLoading from "../Loading/index.js";
const FetchLoading = getLoading(loadingId);
function Routes(props) {
    return useRoutes(props.router);
}
function ScrollToTop() {
    const { pathname } = useLocation();
    useEffect(() => {
        window.scrollTo(0, 0);
    }, [pathname]);
    return null;
}
function RouteContainer(props) {
    const { pathname, search } = useLocation();
    useEffect(() => {
        const cb = async () => {
            const matchedBranch = getMatchedBranch(props.pageRouter, pathname);
            if (!matchedBranch) {
                window.location.reload();
                return;
            }
            const matchedPageCom = getMatchedComponent(matchedBranch);
            const { preloadData, pageReducerName, pageComName } = await getInitialData(matchedPageCom, {
                baseUrl: pathname,
                query: queryString.parse(search),
            });
            Object.keys(preloadData).forEach(name => {
                props.dispatch({ type: `${pageInit}${name}`, data: preloadData[name] });
            });
            if (preloadData[pageReducerName] && preloadData[pageReducerName].title) {
                document.title = preloadData[pageReducerName].title;
            }
        };
        cb();
    }, [pathname, search]);
    return (React.createElement(React.Fragment, null,
        React.createElement(ScrollToTop, null),
        React.createElement(Routes, { router: props.pageRouter }),
        React.createElement("div", { className: "loading-container" },
            React.createElement(FetchLoading, null))));
}
export default reduxConnect()(RouteContainer);
