import { combineReducers } from "redux";
import { pageInit } from "../../config/index.js";
import { getStore } from "../getStore.js";
import { fetchWithRequestObject } from "../fetch.js";
import getLogger from "../utils/logger.js";
import appState from "./state.js";
const logger = getLogger().getLogger("iso/libs/metaCollector");
const finalReducer = {};
const reducerComponentMap = {};
function finalPageReducer(pageReducer, reducerName) {
    return (state, action) => {
        if (action.type.startsWith(pageInit)) {
            const rg = new RegExp("^" + pageInit + "(.+)$");
            const matched = rg.exec(action.type);
            if (matched && matched[1] === reducerName) {
                return {
                    ...state,
                    ...action.data,
                };
            }
            return state;
        }
        return pageReducer(state, action);
    };
}
export function registerRedux(reducer, reducerName, component, subComponents) {
    finalReducer[reducerName] = finalPageReducer(reducer, reducerName);
    reducerComponentMap[reducerName] = { component, reducer, subComponents };
    if (appState.isClientBootstraped) {
        const store = getStore();
        store.replaceReducer(combineReducers(finalReducer));
    }
}
export function getRootReducer() {
    return combineReducers(finalReducer);
}
export function getMatchedComponent(matchedRoute) {
    if (matchedRoute.route.element) {
        return {
            params: matchedRoute.params,
            element: matchedRoute.route.element,
            pageComName: matchedRoute.route.name,
        };
    }
    return null;
}
export async function getInitialData(matchedPageCom, request) {
    if (matchedPageCom && matchedPageCom.element) {
        const component = matchedPageCom.element;
        const { preloadData, pageReducerName } = await collectPreloadData(reducerComponentMap, component, request, {
            body: request.body || {},
            query: request.query || {},
            params: matchedPageCom.params ? {
                ...matchedPageCom.params,
            } : {},
        });
        return {
            preloadData,
            pageReducerName,
            pageComName: matchedPageCom.pageComName,
        };
    }
    else {
        return {
            preloadData: {},
            pageComName: "",
            pageReducerName: "",
        };
    }
}
async function getSinglePreloadData(component, reducer, request, params) {
    let initialData = {};
    if (component.getInitialData) {
        try {
            initialData = await component.getInitialData(fetchWithRequestObject(request), params);
        }
        catch (e) {
            logger.warn("业务方未捕获初始数据获取失败的异常 ", e);
        }
    }
    else {
        logger.info("component dont have getInitialData method.");
    }
    let initialState = reducer(undefined, {});
    initialState = typeof initialState === "object" ? initialState : {};
    return {
        ...initialState,
        ...initialData,
    };
}
async function collectPreloadData(rcMap, targetComponent, request, params) {
    let preloadData = {};
    let pageReducerName = "";
    await Promise.all(Object.keys(rcMap)
        .map(async (reducerName) => {
        const isCompEqual = rcMap[reducerName].component === targetComponent;
        const isCompTypeEqual = rcMap[reducerName].component === targetComponent.type;
        if (isCompEqual || isCompTypeEqual) {
            pageReducerName = reducerName;
            const reducer = rcMap[reducerName].reducer;
            const subComponents = rcMap[reducerName].subComponents;
            const currComponent = rcMap[reducerName].component;
            preloadData[reducerName] = await getSinglePreloadData(currComponent, reducer, request, params);
            if (subComponents) {
                await Promise.all(subComponents.map(async (subComponent) => {
                    const subComPreloadData = await collectPreloadData(rcMap, subComponent, request, params);
                    preloadData = {
                        ...preloadData,
                        ...subComPreloadData.preloadData,
                    };
                }));
            }
        }
    }));
    return { preloadData, pageReducerName };
}
