/// <reference types="node" />
import BodyParser from "body-parser";
import { type CorsOptions, type CorsOptionsDelegate } from "cors";
import Express from "express";
import { type HelmetOptions } from "helmet";
import { Options as ProxyOptions } from "http-proxy-middleware";
import * as net from "net";
import ServeStatic from "serve-static";
interface IBodyParserOption {
    raw?: boolean | BodyParser.Options;
    json?: boolean | BodyParser.OptionsJson;
    text?: boolean | BodyParser.OptionsText;
    urlencoded?: boolean | BodyParser.OptionsUrlencoded;
}
interface IStaticOption {
    path: string[];
    directory: string;
    staticOption?: ServeStatic.ServeStaticOptions;
    isInternal?: true;
}
interface IServerProxyRouter {
    [path: string]: string;
}
interface IProxyConfig extends ProxyOptions {
    changeOriginReferer?: boolean;
}
interface IServerProxyOption {
    path: string;
    config: IProxyConfig;
}
interface ISingleRouteCorsOption {
    path: string;
    corsOptions: CorsOptions | CorsOptionsDelegate;
}
interface IWebServerOption {
    notSSR?: boolean;
    access?: any;
    compress?: boolean;
    cookieParser?: boolean;
    bodyParser?: boolean | IBodyParserOption;
    headers?: string;
    hostname?: "local-ip" | "local-ipv4" | "local-ipv6";
    port?: number;
    proxy?: string | IServerProxyRouter | IServerProxyOption[];
    middleware?: any;
    static?: IStaticOption[];
    secureHeaderOptions?: HelmetOptions | false;
    corsOptions?: CorsOptions | CorsOptionsDelegate | ISingleRouteCorsOption[];
    onServerClosed?: () => void;
    onListening?: (server: net.Server) => void;
}
declare class WebServer {
    private state;
    private options;
    private routers;
    protected name: string;
    server: net.Server;
    app: Express.Express;
    constructor(options?: IWebServerOption);
    private checkServerStarted;
    private setupApp;
    private createServer;
    private getInteralStatic;
    private setCloseHandle;
    private setHealthCheck;
    private errorEventHandler;
    private listen;
    private setRouter;
    private setMiddleware;
    private setNotSSRQueryFeature;
    private setupAccessFeature;
    private setupMiddleware;
    private setupCorsFeature;
    private setupHelmetFeature;
    private setupCompressFeature;
    private setupCookieParserFeature;
    private changeProxyReqReferer;
    private getProxyMW;
    private setupProxyFeature;
    private setupBodyParserFeature;
    private setupHeadersFeature;
    private setContentHeaders;
    private setupStaticFeature;
    private ready;
    bootstrapAsync(): Promise<this>;
    useRouter(router: any): this;
    close(cb?: () => void): void;
    protected bootstrapAsyncBefore(): Promise<void>;
}
export default WebServer;
