import Express from "express";
import { matchRoutes } from "react-router-dom";
import getPathForRouterMatch from "../../utils/getPathForRouterMatch.js";
import getLogger from "../../utils/logger.js";
import { getErrorPageRenderString, getResponsePage } from "../pageRender.js";
import state from "../state.js";
import Router from "./index.js";
const logger = getLogger().getLogger("server/libs/router/pageRouter");
class PageRouter extends Router {
    pageRouter;
    router = Express.Router();
    constructor(pageRouter, meta) {
        super();
        state.meta = meta;
        this.pageRouter = pageRouter;
    }
    setRouter() {
        this.router.use(async (req, res) => {
            res.on('finish', () => {
                logger.info("响应完成.");
            });
            try {
                const originalUrl = req.originalUrl;
                const path = getPathForRouterMatch(originalUrl);
                const matchedRoute = matchRoutes(this.pageRouter, path);
                logger.info("originalUrl: ", originalUrl);
                logger.info("path: ", path);
                logger.info("matchedRoute: ", matchedRoute);
                if (!matchedRoute) {
                    logger.warn(`not match any router branch. originalUrl: ${originalUrl}, path: ${path}`);
                    res.status(404).end();
                    return;
                }
                res.setHeader("Content-Type", "text/html; charset=utf-8");
                res.write("<!DOCTYPE html>");
                logger.info("match router branch.");
                const htmlPipableStream = await getResponsePage(req, res, this.pageRouter, matchedRoute[matchedRoute.length - 1]);
                logger.info("stream准备完成，准备响应页面给客户端");
                htmlPipableStream.pipe(res);
            }
            catch (err) {
                res.end(getErrorPageRenderString(), "utf-8");
                logger.error("服务端路由处理时出现异常: ", err.message, " ; stack: ", err.stack);
            }
        });
    }
}
export default PageRouter;
